//
//  LWConfigViewController.m
//  APConfig
//
//  Created by wen on 2018/8/2.
//  Copyright © 2018年 wen. All rights reserved.
//

#import "LWConfigViewController.h"
#import <net/if.h>
#import <ifaddrs.h>
#import <SystemConfiguration/CaptiveNetwork.h>
#import "LWControlHeader.h"
#import "HttpRequest.h"
#import "LoadingView.h"
#import "AKAlertView.h"
#import "Udpproxy.h"
#import <NetworkExtension/NEHotspotConfigurationManager.h>
#import "MBProgressHUD+MJ.h"

#define V8_RANDOM_NUM          0xAA
#define ActivityColor LWUIColorFromRGB(0x08a6f2)
static NSString *const APInfo_SSID = @"APInfo_SSID";
static NSString *const APInfo_pwd  = @"APInfo_pwd";

@interface LWConfigViewController ()
@property (weak, nonatomic) IBOutlet UITextField *apPwdTextField;
@property (weak, nonatomic) IBOutlet UITextField *profixTextField;
@property (weak, nonatomic) IBOutlet UILabel *currentSSIDLabel;
@property (weak, nonatomic) IBOutlet UIButton *nextBtn;
@property (weak, nonatomic) IBOutlet UIButton *scanBtn;
/**设备热点密码*/
@property (weak, nonatomic) IBOutlet UITextField *devicePwdTextField;

@property (nonatomic, strong) NSString *currentSSID;

/**设备wifi热点**/
@property (nonatomic, strong) NSString *deviceSSID;

@property (nonatomic, strong) LoadingView *loadV;
@property (nonatomic, strong) AKAlertView *alertV;
@property (nonatomic, strong) Udpproxy * udp;
@property (nonatomic, strong) NSTimer *timer;

@property (nonatomic, assign) BOOL isConnectDevcie;
//loadView
@property (weak, nonatomic) IBOutlet UIView *whiteView;
@property (weak, nonatomic) IBOutlet UIActivityIndicatorView *activityV;
@property (weak, nonatomic) IBOutlet UIButton *closeBtn;

@property (strong, nonatomic) UIAlertController *alertVC;


/**开始扫描*/
@property (nonatomic, assign) BOOL isStartScan;
@end

@implementation LWConfigViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    self.title = @"AP Distribution network";
    [self configContentView];
}

-(void)viewWillAppear:(BOOL)animated{
    [super viewWillAppear:animated];
    [[NSNotificationCenter defaultCenter ] addObserver:self selector:@selector(updateUI) name:@"KOnWillEnterForeground" object:nil];
    self.scanBtn.hidden = YES;
    [self configContentView];
    [self hideLoadView];
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    NSString *apSSID = [userDefaults objectForKey:APInfo_SSID];
    NSString *apPwd  = [userDefaults objectForKey:APInfo_pwd];

    if (apSSID.length > 0) {
        self.profixTextField.text = apSSID;
    }else{
        self.profixTextField.text = @"hiflying_softap";
    }
    
    if (apPwd.length > 0) {
        self.devicePwdTextField.text = apPwd;
    }

}

-(void)viewDidDisappear:(BOOL)animated{
    [super viewDidDisappear:animated];
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    [self.timer invalidate];
    self.timer = nil;
    [self hideLoadView];
    
}

-(void)updateUI{
    [self configContentView];
    [self.alertVC removeFromParentViewController];
}

-(void)configContentView{
    NSString *preStr = self.profixTextField.text;
    id info = [self fetchSSIDInfo];
    self.currentSSIDLabel.text = [NSString stringWithFormat:@"Current Wi-Fi Connection:%@",[info objectForKey:@"SSID"]];
    self.currentSSID = [info objectForKey:@"SSID"];
    if ([self.currentSSID containsString:preStr]) {
        //当前手机连接上了模块热点
        dispatch_async(dispatch_get_main_queue(), ^{
            [self.nextBtn setTitle:@"Start the configuration" forState:UIControlStateNormal];
            self.scanBtn.hidden = YES;
        });
    }else{
        dispatch_async(dispatch_get_main_queue(), ^{
            [self.nextBtn setTitle:@"To set up" forState:UIControlStateNormal];
            self.nextBtn.hidden = NO;
        });
    }
    
    if ([self.currentSSID containsString:self.netSSID]&&self.isStartScan) {
        //当发现手机已经连接回了之前配置的wifi
        [self scan];
    }
}


#pragma mark - UI事件
- (IBAction)nextBtnDidClicked:(id)sender {
    //配网设备
    [self.profixTextField resignFirstResponder];
    [self.devicePwdTextField resignFirstResponder];
    if (![self isWiFiEnabled]) {
        //没有打开wifi开关
        [self presentAlertWithMessage:@"Please open the wifi"];
        return;
    }
   self.currentSSID = [self getCurrentWifi];;
    
    NSString *preStr = self.profixTextField.text;
    if ([self.currentSSID containsString:preStr]) {
        //当前手机连接上了设备热点
        [self apConfigWithNetSSID:self.netSSID withNetPwd:self.netPwd withSoftApSSID:self.profixTextField.text];
    }else{
        
        //进入手机设置界面
//        [self SkipSettingVC];
        // 2018.11.19  改为自动连接设备热点，并且配网成功后也是自动连回之前的wifi热点
        //保存到本地
        NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
        [userDefaults setObject:self.profixTextField.text forKey:APInfo_SSID];
        [userDefaults setObject:self.devicePwdTextField.text forKey:APInfo_pwd];
        [userDefaults synchronize];
        
        NEHotspotConfiguration *hotspotConfig;
        if (self.devicePwdTextField.text.length > 0) {
            //有密码
             hotspotConfig = [[NEHotspotConfiguration alloc] initWithSSID:self.profixTextField.text passphrase:self.devicePwdTextField.text isWEP:NO];
        }else{
            //没有密码
            hotspotConfig = [[NEHotspotConfiguration alloc] initWithSSID:self.profixTextField.text];
        }
        
        
        [self showLoadView];
        __weakSelf(self);
        _isConnectDevcie = YES;
        [self performSelector:@selector(checkLoading) withObject:nil afterDelay:6];
        [[NEHotspotConfigurationManager sharedManager] applyConfiguration:hotspotConfig completionHandler:^(NSError * _Nullable error) {
            
            LWLog(@"%@", error);
            weakSelf.isConnectDevcie = NO;
            dispatch_async(dispatch_get_main_queue(), ^{
                id info = [weakSelf fetchSSIDInfo];
                weakSelf.currentSSIDLabel.text = [NSString stringWithFormat:@"Current Wi-Fi Connection:%@",[info objectForKey:@"SSID"]];
            });
            NSString *currentSSid = [weakSelf getCurrentWifi];
            if ([currentSSid isEqualToString:self.profixTextField.text]) {
                //配网成功
                dispatch_async(dispatch_get_main_queue(), ^{
                    [weakSelf hideLoadView];
                });
                weakSelf.currentSSID = self.profixTextField.text;
                dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.3 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
                    [weakSelf hideLoadView];
                    [weakSelf apConfigWithNetSSID:self.netSSID withNetPwd:weakSelf.netPwd withSoftApSSID:self.profixTextField.text];
                });
            }else{
                //连接失败
                if (error.description.length > 0) {
                    dispatch_async(dispatch_get_main_queue(), ^{
                        [weakSelf hideLoadView];
                        [weakSelf presentAlertWithMessage:error.description];
                    });
                }
                
            }
        }];
    }
}

- (IBAction)scanBtnDidClicked:(id)sender {
    //扫描局域网下设备
    if (![self isWiFiEnabled]) {
        //没有打开wifi开关
        [self presentAlertWithMessage:@"Please open the wifi"];
        return;
    }
    NSString *currentSSid = [self getCurrentWifi];
    if ([currentSSid isEqualToString:self.netSSID]) {
        //是否已经连接上了之前的wifi
        [self scan];
        return;
    }
    
    dispatch_async(dispatch_get_main_queue(), ^{
        [self showLoadView];
    });
    // 2018.11.19 自动连接到之前配置的wifi热点
    __weakSelf(self);
    NEHotspotConfiguration *hotspotConfig = [[NEHotspotConfiguration alloc] initWithSSID:self.netSSID passphrase:self.netPwd isWEP:NO];
    [[NEHotspotConfigurationManager sharedManager] applyConfiguration:hotspotConfig completionHandler:^(NSError * _Nullable error) {
        LWLog(@"%@", error);
        
        NSString *currentSSid = [self getCurrentWifi];
        dispatch_async(dispatch_get_main_queue(), ^{
            id info = [weakSelf fetchSSIDInfo];
            weakSelf.currentSSIDLabel.text = [NSString stringWithFormat:@"Current Wi-Fi Connection:%@",[info objectForKey:@"SSID"]];
        });
        if ([currentSSid isEqualToString:self.profixTextField.text]) {
            //连接上了设备热点
            dispatch_async(dispatch_get_main_queue(), ^{
                [weakSelf hideLoadView];
            });
            
        }else if([currentSSid isEqualToString:self.netSSID]){
            
            dispatch_async(dispatch_get_main_queue(), ^{
                [weakSelf hideLoadView];
                [weakSelf scan];
            });
        }else{
            //连接失败
            if (error.description.length > 0) {
                dispatch_async(dispatch_get_main_queue(), ^{
                    [weakSelf hideLoadView];
                    [weakSelf presentAlertWithMessage:error.description];
                });
            }
            
        }
    }];
    
    LWLog(@"thread == %@",[NSThread currentThread]);
}

-(void)scan{
    
    [self.timer invalidate];
    self.timer = nil;
    [self.udp close];
    
    self.udp = [Udpproxy shareInstence];
    [self.udp CreateBindSocket];
    [self showLoadView];
    __weakSelf(self);
    [self.udp sendSmartLinkFind];
    self.timer = [NSTimer scheduledTimerWithTimeInterval:1.0f repeats:YES block:^(NSTimer * _Nonnull timer) {
        LWLog(@"搜索局域网设备--%@ 线程：%@",[NSDate date],[NSThread currentThread]);
        [weakSelf.udp sendSmartLinkFind];
        HFSmartLinkDeviceInfo * dev = [weakSelf.udp recv:V8_RANDOM_NUM];
        if (dev) {
            dispatch_async(dispatch_get_main_queue(), ^{
//                [weakSelf hideLoadView];
                [weakSelf.activityV stopAnimating];
                weakSelf.whiteView.hidden = YES;
                [weakSelf presentAlertWithMessage:[NSString stringWithFormat:@"MAC:%@\n ip:%@",dev.mac,dev.ip]];
            });
            [weakSelf.timer invalidate];
            weakSelf.timer = nil;
        }
    }];
    [[NSRunLoop mainRunLoop]addTimer:self.timer forMode:NSRunLoopCommonModes];
}

- (IBAction)closeBtnDidClicked:(id)sender {
    //关闭loading 弹框
    __weakSelf(self);
    dispatch_async(dispatch_get_main_queue(), ^{
        [weakSelf.activityV stopAnimating];
        weakSelf.whiteView.hidden = YES;
    });
}

-(void)timerRun{
    LWLog(@"搜索局域网设备--%@ 线程：%@",[NSDate date],[NSThread currentThread]);
    [self.udp sendSmartLinkFind];
    HFSmartLinkDeviceInfo * dev = [self.udp recv:V8_RANDOM_NUM];
    if (dev) {
        dispatch_async(dispatch_get_main_queue(), ^{
            [self hideLoadView];
            [self presentAlertWithMessage:[NSString stringWithFormat:@"MAC:%@\n ip:%@",dev.mac,dev.ip]];
        });
        [self.timer invalidate];
        self.timer = nil;
    }
}

-(void)hideLoadView{
    __weakSelf(self);
    dispatch_async(dispatch_get_main_queue(), ^{
        [weakSelf.activityV stopAnimating];
        weakSelf.whiteView.hidden = YES;
    });
}

-(void)showLoadView{
    
    __weakSelf(self);
    dispatch_async(dispatch_get_main_queue(), ^{
        weakSelf.whiteView.hidden = NO;
        weakSelf.whiteView.layer.cornerRadius = 5;
        weakSelf.whiteView.layer.masksToBounds = YES;
        [weakSelf.activityV setColor:ActivityColor];
        [weakSelf.activityV startAnimating];

    });
}


-(void)presentAlertWithMessage:(NSString *)message{
    UIAlertController *alert = [UIAlertController alertControllerWithTitle:@"" message:message preferredStyle:UIAlertControllerStyleAlert];
    UIAlertAction *centain = [UIAlertAction actionWithTitle:@"sure" style:UIAlertActionStyleDefault handler:nil];
    [alert addAction:centain];
    self.alertVC = alert;
    [self presentViewController:alert animated:YES completion:nil];
}


#pragma mark - method
-(void)apConfigWithNetSSID:(NSString *)netSSID
                withNetPwd:(NSString *)netPwd
            withSoftApSSID:(NSString *)softApSSID{

    [self showLoadView];
    __weakSelf(self);
    [HttpRequest connectModuleWithWiFiPwd:netPwd withNetSSID:netSSID withSSID:softApSSID didLoadData:^(NSDictionary *result, NSError *err) {
        if (!err) {
            NSInteger suc = [result[@"RC"] integerValue];
            if (suc == 0) {
                //成功
                weakSelf.deviceSSID = softApSSID;
                [weakSelf setDeviceRestartWithDeviceSSID:softApSSID];
                //保存wifi名跟密码
                [weakSelf saveNetPWD];
    
            }else{
                //失败
                dispatch_async(dispatch_get_main_queue(), ^{
                    [weakSelf hideLoadView];
                    id info = [weakSelf fetchSSIDInfo];
                    weakSelf.currentSSIDLabel.text = [NSString stringWithFormat:@"Current Wi-Fi Connection:%@",[info objectForKey:@"SSID"]];
                });
            }
        }else{
            LWLog(@"发送配置请求失败");
            dispatch_async(dispatch_get_main_queue(), ^{

                [weakSelf hideLoadView];
                id info = [weakSelf fetchSSIDInfo];
                weakSelf.currentSSIDLabel.text = [NSString stringWithFormat:@"Current Wi-Fi Connection:%@",[info objectForKey:@"SSID"]];
            });
        }
    }];
}

-(void)setDeviceRestartWithDeviceSSID:(NSString *)ssid{
    
    __weakSelf(self);
    [HttpRequest setRestartCommandwithSSID:ssid didLoadData:^(NSDictionary *result, NSError *err) {
        if (!err) {
            NSInteger suc = [result[@"RC"] integerValue];
            if (suc == 0) {
                //成功
                dispatch_async(dispatch_get_main_queue(), ^{
                    [weakSelf hideLoadView];
                    [weakSelf.alertV dismiss];
                    
                    dispatch_async(dispatch_get_main_queue(), ^{
                        [weakSelf showLoadView];
                    });
                    
                    // 2018.11.19 自动连接到之前配置的wifi热点
                    NEHotspotConfiguration *hotspotConfig = [[NEHotspotConfiguration alloc] initWithSSID:weakSelf.netSSID passphrase:weakSelf.netPwd isWEP:NO];
                    [[NEHotspotConfigurationManager sharedManager] applyConfiguration:hotspotConfig completionHandler:^(NSError * _Nullable error) {
                        LWLog(@"%@", error);
                        
                        NSString *currentSSid = [weakSelf getCurrentWifi];
                        dispatch_async(dispatch_get_main_queue(), ^{
                            id info = [weakSelf fetchSSIDInfo];
                            weakSelf.currentSSIDLabel.text = [NSString stringWithFormat:@"Current Wi-Fi Connection:%@",[info objectForKey:@"SSID"]];
                        });
                        if ([currentSSid isEqualToString:self.profixTextField.text]) {
                            //连接上了设备热点
                            dispatch_async(dispatch_get_main_queue(), ^{
                                [weakSelf hideLoadView];
                            });
                            
                        }else if([currentSSid isEqualToString:self.netSSID]){
                            //连接上了要配置的wifi
                            dispatch_async(dispatch_get_main_queue(), ^{
                                [weakSelf hideLoadView];
                                [weakSelf scan];
                            });
                        }else{
                            //连接失败
                            if (error.description.length > 0) {
                                dispatch_async(dispatch_get_main_queue(), ^{
                                    [weakSelf hideLoadView];
                                    [weakSelf presentAlertWithMessage:error.description];
                                });
                            }
                            
                        }
                    }];
                    weakSelf.scanBtn.hidden = NO;
                    weakSelf.isStartScan = YES;
                });
            }else{
                //失败
                 dispatch_async(dispatch_get_main_queue(), ^{
                     [weakSelf hideLoadView];
                     id info = [weakSelf fetchSSIDInfo];
                     weakSelf.currentSSIDLabel.text = [NSString stringWithFormat:@"Current Wi-Fi Connection:%@",[info objectForKey:@"SSID"]];
                 });
            }
        }else{
            LWLog(@"发送重启请求失败");
            dispatch_async(dispatch_get_main_queue(), ^{
                [weakSelf hideLoadView];
                id info = [weakSelf fetchSSIDInfo];
                weakSelf.currentSSIDLabel.text = [NSString stringWithFormat:@"Current Wi-Fi Connection:%@",[info objectForKey:@"SSID"]];
            });
        }
    }];
}

-(void)saveNetPWD{
    [[NSUserDefaults standardUserDefaults]setObject:self.netPwd forKey:self.netSSID];
    [[NSUserDefaults standardUserDefaults] synchronize];
}
-(void)checkLoading{
    if (_isConnectDevcie) {
        [self hideLoadView];
    }
}

/**
 *  判断手机是否打开了wifi开关
 */
- (BOOL) isWiFiEnabled {
    NSCountedSet * cset = [[NSCountedSet alloc] init];
    struct ifaddrs *interfaces;
    if( ! getifaddrs(&interfaces) ) {
        for( struct ifaddrs *interface = interfaces; interface; interface = interface->ifa_next) {
            if ( (interface->ifa_flags & IFF_UP) == IFF_UP ) {
                [cset addObject:[NSString stringWithUTF8String:interface->ifa_name]];
                
            }
        }
    }
    return [cset countForObject:@"awdl0"] > 1 ? YES : NO;
}


#pragma mark- GetSSID
- (id)fetchSSIDInfo {
    NSArray *ifs = (__bridge_transfer id)CNCopySupportedInterfaces();
    id info = nil;
    for (NSString *ifnam in ifs) {
        info = (__bridge_transfer id)CNCopyCurrentNetworkInfo((__bridge CFStringRef)ifnam);
        if (info && [info count]) { break; }
    }
    return info;
}

-(NSString *)getCurrentWifi{
    NSString *ssid = nil;
    NSArray *ifs = (__bridge   id)CNCopySupportedInterfaces();
    for (NSString *ifname in ifs) {
        NSDictionary *info = (__bridge id)CNCopyCurrentNetworkInfo((__bridge CFStringRef)ifname);
        if (info[@"SSID"])
        {
            ssid = info[@"SSID"];
        }
    }
    return ssid;

}

-(void)touchesBegan:(NSSet<UITouch *> *)touches withEvent:(UIEvent *)event{
    [self.profixTextField resignFirstResponder];
    [self.devicePwdTextField resignFirstResponder];
}



- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

@end
