//
//  LWWiFiViewController.m
//  APConfig
//
//  Created by wen on 2018/7/31.
//  Copyright © 2018年 wen. All rights reserved.
//

#import "LWWiFiViewController.h"
#import <SystemConfiguration/CaptiveNetwork.h>
#import "HttpRequest.h"
#import "MBProgressHUD+MJ.h"
#import "LWConfigViewController.h"
#import <NetworkExtension/NEHotspotConfigurationManager.h>
#import "LWControlHeader.h"
#import "LWBasicNetWorkTools.h"
#import "LoadingView.h"

@interface LWWiFiViewController ()

@property (weak, nonatomic) IBOutlet UILabel *versionLabel;
@property (weak, nonatomic) IBOutlet UILabel *ssidLabel;
@property (weak, nonatomic) IBOutlet UITextField *wifiPwdTextField;
@property (weak, nonatomic) IBOutlet UITextField *ssidPrefixTextField;
@property (weak, nonatomic) IBOutlet UITextField *apPwdTextField;
@property (weak, nonatomic) IBOutlet UIButton *nextBtn;
@property (weak, nonatomic) IBOutlet UILabel *errorLabel;

/**模块热点前缀*/
@property (nonatomic, strong) NSString *ssidPrefix;
/**模块要连的wifi*/
@property (nonatomic, strong) NSString *netSSID;
/**手机当前连接的wifi*/
@property (nonatomic, strong) NSString *currentSSID;
/**模块要连的wifi密码*/
@property (nonatomic, strong) NSString *wifiPwd;
/**模块热点密码*/
@property (nonatomic, strong) NSString *deviceWiFiPwd;
@end

@implementation LWWiFiViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    [self.wifiPwdTextField becomeFirstResponder];
    self.ssidPrefix = @"hiflying_softap";
    self.ssidPrefixTextField.text = self.ssidPrefix;
    [self configContentView];
    
    self.versionLabel.text = [NSString stringWithFormat:@"version:%@.%@",[[[NSBundle mainBundle]infoDictionary] objectForKey:@"CFBundleShortVersionString"],[[[NSBundle mainBundle]infoDictionary] objectForKey:@"CFBundleVersion"]];

}

-(void)viewWillAppear:(BOOL)animated{
    [super viewWillAppear:animated];
    [[NSNotificationCenter defaultCenter ] addObserver:self selector:@selector(updateUI) name:@"KOnWillEnterForeground" object:nil];

    [self configContentView];
    
    
}

-(void)viewDidDisappear:(BOOL)animated{
    [super viewDidDisappear:animated];
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

-(void)configContentView{
    id info = [self fetchSSIDInfo];
    self.ssidLabel.text = [info objectForKey:@"SSID"];
    if ([self.ssidLabel.text containsString:@"hiflying_softap"]) {
        [self.nextBtn setTitle:@"To set up" forState:UIControlStateNormal];
        self.errorLabel.text = @"Select the WIFi the device will connect to";
    }else{
        [self.nextBtn setTitle:@"Next" forState:UIControlStateNormal];
        self.errorLabel.text = @"";
    }
    
    if (self.ssidLabel.text) {
        NSString *pwd = [[NSUserDefaults standardUserDefaults] objectForKey:self.ssidLabel.text];
        if (pwd) {
            self.wifiPwdTextField.text = pwd;
        }
    }
}

#pragma mark - UI事件
- (IBAction)startBtnDidClicked:(id)sender {
//    
//    NSArray *testArray = @[@"",@"2"];
//    testArray[4];
    self.netSSID = self.ssidLabel.text ;
    self.wifiPwd = self.wifiPwdTextField.text;
    self.deviceWiFiPwd = self.apPwdTextField.text;
    if ([self.netSSID containsString:@"hiflying_softap"]) {
        
        self.errorLabel.text = @"Please select the wifi hotspot that the hardware needs to connect to first";
        [self performSelector:@selector(skipSettingVC) withObject:nil afterDelay:0.35];

    }else{
        
        if (self.netSSID.length == 0) {
            
            self.errorLabel.text = @"Connect your phone to wifi";
            [self performSelector:@selector(skipSettingVC) withObject:nil afterDelay:0.35];
            return;
        }else{
            if (self.deviceWiFiPwd.length >0 ) {
                
                if (self.deviceWiFiPwd.length >= 8) {
                    [self pushConfigVC];
                }else{
                    self.errorLabel.text = @"Please enter a device hotspot password greater than 8 bits";
                }
                
            }else{
                
                [self pushConfigVC];

            }
            
        }
    }
}

-(void)skipSettingVC{
    [[UIApplication sharedApplication]openURL:[NSURL URLWithString:UIApplicationOpenSettingsURLString]];
}


-(void)pushConfigVC{
    UIStoryboard *wifiStoryboard = [UIStoryboard storyboardWithName:@"LWWiFiStoryboard" bundle:nil];
    LWConfigViewController *configVC = [wifiStoryboard instantiateViewControllerWithIdentifier:@"LWConfigViewController"];
    configVC.netSSID = self.netSSID;
    configVC.netPwd  = self.wifiPwd;
    [self.navigationController pushViewController:configVC animated:YES];
}




-(void)updateUI{
    [self configContentView];
}


#pragma mark- GetSSID
- (id)fetchSSIDInfo {
    NSArray *ifs = (__bridge_transfer id)CNCopySupportedInterfaces();
    id info = nil;
    for (NSString *ifnam in ifs) {
        info = (__bridge_transfer id)CNCopyCurrentNetworkInfo((__bridge CFStringRef)ifnam);
        if (info && [info count]) { break; }
    }
    return info;
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
}


-(void)touchesBegan:(NSSet<UITouch *> *)touches withEvent:(UIEvent *)event{
    [self.wifiPwdTextField resignFirstResponder];
    [self.ssidPrefixTextField resignFirstResponder];
    [self.apPwdTextField resignFirstResponder];
}






/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

@end
