package com.hiflying.aplink.demo;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.location.LocationManager;
import android.net.wifi.WifiInfo;
import android.os.Build;
import android.os.Bundle;
import android.provider.Settings;
import android.text.Editable;
import android.text.TextWatcher;
import android.util.Log;
import android.view.View;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

import com.alibaba.fastjson.JSON;
import com.hiflying.aplink.LinkedModule;
import com.hiflying.aplink.LinkingError;
import com.hiflying.aplink.LinkingProgress;
import com.hiflying.aplink.OnLinkListener;
import com.hiflying.aplink.v1.ApLinker;

import androidx.annotation.NonNull;
import permissions.dispatcher.NeedsPermission;
import permissions.dispatcher.OnNeverAskAgain;
import permissions.dispatcher.OnPermissionDenied;
import permissions.dispatcher.RuntimePermissions;

@RuntimePermissions
public class MainActivity extends Activity implements OnLinkListener {

    private static final String TAG = "ApLinker:MainActivity";
    private static final String KEY_AP_SSID = "ap_ssid";
    private static final String KEY_AP_PASSWORD = "ap_password";
    private static final String KEY_SSID_FORMAT = "ssid_%s";

    private EditText mSsidEditText;
    private EditText mPasswordEditText;
    private EditText mApSsidEditText;
    private EditText mApPasswordEditText;
    private Button mLinkButton;
    private TextView mMessageTextView;

    private ApLinker mApLinker;
    private boolean mWifiConnectedBeforeLink;
    private ProgressDialog mProgressDialog;
    private ProgressDialog mCancelingDialog;
    private AlertDialog mAlertDialog;
    private SharedPreferences mSharedPreferences;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
        mSharedPreferences = getSharedPreferences(TAG, MODE_PRIVATE);

        setContentView(R.layout.activity_main);
        setupViews();

        mApLinker = ApLinker.getInstance(this);
        mApLinker.init();
        mApLinker.setOnLinkListener(this);
    }

    @Override
    protected void onStart() {
        super.onStart();
        checkLocationProvider();
        MainActivityPermissionsDispatcher.requestPermissionsWithPermissionCheck(this);
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        mApLinker.destroy();
    }
    private void setupViews() {

        mSsidEditText = (EditText) findViewById(R.id.ssid);
        mPasswordEditText = (EditText) findViewById(R.id.password);
        mApSsidEditText = (EditText) findViewById(R.id.ap_ssid);
        mApPasswordEditText = (EditText) findViewById(R.id.ap_password);
        mMessageTextView = (TextView) findViewById(R.id.message);
        mLinkButton = (Button) findViewById(R.id.link);

        setEditTexWithSharedPreferences();

        TextWatcher textWatcher = new TextWatcher() {

            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {

            }

            @Override
            public void afterTextChanged(Editable s) {

                if(!mApLinker.isSmartLinking() && mWifiConnectedBeforeLink &&
                        !mSsidEditText.getText().toString().isEmpty() && !mApSsidEditText.getText().toString().trim().isEmpty()) {
                    mLinkButton.setEnabled(true);
                }else {
                    mLinkButton.setEnabled(false);
                }
            }
        };

        mSsidEditText.addTextChangedListener(textWatcher);
        mPasswordEditText.addTextChangedListener(textWatcher);
        mApSsidEditText.addTextChangedListener(textWatcher);
        mApPasswordEditText.addTextChangedListener(textWatcher);
        mLinkButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

                SharedPreferences.Editor editor = mSharedPreferences.edit();
                editor.putString(String.format(KEY_SSID_FORMAT, mSsidEditText.getText().toString()), mPasswordEditText.getText().toString());
                editor.putString(KEY_AP_SSID, mApSsidEditText.getText().toString().trim());
                editor.putString(KEY_AP_PASSWORD, mApPasswordEditText.getText().toString());
                editor.commit();

                mMessageTextView.setText(null);
                mApLinker.setSsid(mSsidEditText.getText().toString());
                mApLinker.setPassword(mPasswordEditText.getText().toString());
                mApLinker.setApSsid(mApSsidEditText.getText().toString().trim());
                mApLinker.setApPassword(mApPasswordEditText.getText().toString());
                try {
                    mApLinker.start();
                    mLinkButton.setEnabled(false);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        });

        mCancelingDialog = new ProgressDialog(this);
        mCancelingDialog.setMessage(getString(R.string.aplink_canceling));
        mCancelingDialog.setCanceledOnTouchOutside(false);
        mCancelingDialog.setCancelable(false);

        try {
            PackageInfo packageInfo = getPackageManager().getPackageInfo(getPackageName(), PackageManager.GET_ACTIVITIES);
            if (packageInfo != null) {
                ((TextView)findViewById(R.id.version)).setText("version: " + packageInfo.versionName);
            }
        } catch (PackageManager.NameNotFoundException e) {
            e.printStackTrace();
        }
    }

    @NeedsPermission({"android.permission.ACCESS_WIFI_STATE", "android.permission.ACCESS_NETWORK_STATE", "android.permission.ACCESS_COARSE_LOCATION",
            "android.permission.ACCESS_FINE_LOCATION", "android.permission.CHANGE_WIFI_STATE", "android.permission.CHANGE_NETWORK_STATE",
            "android.permission.INTERNET", "android.permission.WAKE_LOCK"})
    public void requestPermissions() {
        Log.d(TAG, "requestPermissions: ");
    }

    @OnPermissionDenied({"android.permission.ACCESS_WIFI_STATE", "android.permission.ACCESS_NETWORK_STATE", "android.permission.ACCESS_COARSE_LOCATION",
            "android.permission.ACCESS_FINE_LOCATION", "android.permission.CHANGE_WIFI_STATE", "android.permission.CHANGE_NETWORK_STATE",
            "android.permission.INTERNET", "android.permission.WAKE_LOCK"})
    public void showPermissionDenied() {
        showAlertDialog(getString(R.string.aplink_permission_denied));
    }


    @OnNeverAskAgain({"android.permission.ACCESS_WIFI_STATE", "android.permission.ACCESS_NETWORK_STATE", "android.permission.ACCESS_COARSE_LOCATION",
            "android.permission.ACCESS_FINE_LOCATION", "android.permission.CHANGE_WIFI_STATE", "android.permission.CHANGE_NETWORK_STATE",
            "android.permission.INTERNET", "android.permission.WAKE_LOCK"})
    public void onPermissionNeverAskAgain() {
        showAlertDialog(getString(R.string.aplink_permission_denied));
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        MainActivityPermissionsDispatcher.onRequestPermissionsResult(this, requestCode, grantResults);
    }

    private void setEditTexWithSharedPreferences() {
        mPasswordEditText.setText(mSharedPreferences.getString(String.format(KEY_SSID_FORMAT, mSsidEditText.getText().toString()), null));
        mApSsidEditText.setText(mSharedPreferences.getString(KEY_AP_SSID, ApLinker.HIFLYING_SOFT_AP_SSID));
        mApPasswordEditText.setText(mSharedPreferences.getString(KEY_AP_PASSWORD, null));
    }

    @Override
    public void onWifiConnectivityChangedBeforeLink(boolean connected, String ssid, WifiInfo wifiInfo) {

        mWifiConnectedBeforeLink = connected;
        if (connected) {
            mSsidEditText.setText(ssid);
            mPasswordEditText.setText(mSharedPreferences.getString(String.format(KEY_SSID_FORMAT, ssid), null));
            mApSsidEditText.setText(mSharedPreferences.getString(KEY_AP_SSID, ApLinker.HIFLYING_SOFT_AP_SSID));
            mApPasswordEditText.setText(mSharedPreferences.getString(KEY_AP_PASSWORD, null));

            mPasswordEditText.setEnabled(true);
            mApSsidEditText.setEnabled(true);
            mApPasswordEditText.setEnabled(true);
            if(!mSsidEditText.getText().toString().trim().isEmpty() && !mApSsidEditText.getText().toString().trim().isEmpty()) {
                mLinkButton.setEnabled(true);
            }else {
                mLinkButton.setEnabled(false);
            }

            if (mMessageTextView.getText().toString().equals(getString(R.string.aplink_no_valid_wifi_connection))) {
                mMessageTextView.setText(null);
            }
        }else {
            mSsidEditText.setText(null);
            mPasswordEditText.setText(null);
            mApSsidEditText.setText(null);
            mApPasswordEditText.setText(null);
            mPasswordEditText.setEnabled(false);
            mApSsidEditText.setEnabled(false);
            mApPasswordEditText.setEnabled(false);
            mLinkButton.setEnabled(false);
            mMessageTextView.setText(R.string.aplink_no_valid_wifi_connection);
        }
    }

    @Override
    public void onLinked(LinkedModule module) {
        Log.i(TAG, "onLinked: " + module);

        updateMessage("onLinked: " + module.getMac());
        showAlertDialog("linked module: " + JSON.toJSONString(module));
    }

    @Override
    public void onFinished() {
        Log.i(TAG, "onFinished");

        updateMessage("onFinished");
        dismissProgressDialog();
        if (mCancelingDialog.isShowing()) {
            mCancelingDialog.dismiss();
        }
        mLinkButton.setEnabled(true);
    }

    @Override
    public void onTimeOut() {
        Log.i(TAG, "onTimeOut");

        updateMessage("onTimeOut");
        showAlertDialog("TIME OUT");
    }

    @Override
    public void onError(LinkingError error) {
        Log.i(TAG, "onError: " + error);

        updateMessage("onError: " + error);
        dismissProgressDialog();
        showAlertDialog(error.name());
    }

    @Override
    public void onProgress(LinkingProgress progress) {
        Log.i(TAG, "onProgress: " + progress);

        updateMessage("onProgress: " + progress);
        showProgressDialog(progress.name());
    }

    private void showProgressDialog(String message) {

        if (mProgressDialog == null) {
            mProgressDialog = new ProgressDialog(this);
            mProgressDialog.setTitle(R.string.aplink_app_name);
            mProgressDialog.setCanceledOnTouchOutside(false);
            mProgressDialog.setCancelable(false);
            mProgressDialog.setButton(DialogInterface.BUTTON_NEGATIVE, getString(android.R.string.cancel), new DialogInterface.OnClickListener() {
                @Override
                public void onClick(DialogInterface dialog, int which) {
                    mApLinker.stop();
                    mCancelingDialog.show();
                }
            });
//            mProgressDialog.setOnDismissListener(new DialogInterface.OnDismissListener() {
//                @Override
//                public void onDismiss(DialogInterface dialog) {
//                    mApLinker.stop();
//                }
//            });
        }
        mProgressDialog.setMessage(message);
        if (!mProgressDialog.isShowing()) {
            mProgressDialog.show();
        }
    }

    private void showAlertDialog(String message) {

        if (mAlertDialog == null) {
            mAlertDialog = new AlertDialog.Builder(this)
                    .setTitle(R.string.aplink_app_name)
                    .setPositiveButton(android.R.string.ok, null)
                    .create();
        }
        mAlertDialog.setMessage(message);
        if (!mAlertDialog.isShowing()) {
            mAlertDialog.show();
        }
    }

    private void dismissProgressDialog() {
        if (mProgressDialog.isShowing()) {
            mProgressDialog.dismiss();
        }
    }

    private void updateMessage(String message) {

        mMessageTextView.setText(mMessageTextView.getText().toString().concat("\n").concat(message));
    }

    private void checkLocationProvider() {

        if(Build.VERSION.SDK_INT >= Build.VERSION_CODES.M){

            LocationManager locManager = (LocationManager)getSystemService(LOCATION_SERVICE);
            if (!locManager.isProviderEnabled(LocationManager.GPS_PROVIDER)) {

                Log.w(TAG, String.format("The android version sdk is %s and its location provider is disabled!", Build.VERSION.SDK_INT));

                new AlertDialog.Builder(this)
                        .setTitle(R.string.aplink_app_name)
                        .setMessage(R.string.aplink_location_prodiver_disabled)
                        .setPositiveButton(android.R.string.ok, new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialog, int which) {

                                Intent intent =  new Intent(Settings.ACTION_LOCATION_SOURCE_SETTINGS);
                                startActivity(intent);
                            }
                        })
                        .create()
                        .show();
            }
        }
    }
}
